package ru.dbotthepony.kstarbound.client.render

import com.google.common.collect.ImmutableMap
import org.apache.logging.log4j.LogManager

enum class RenderLayer {
	BackgroundOverlay,
	BackgroundTile,
	BackgroundTileMod,
	Platform,
	Plant,
	PlantDrop,
	Object,
	PreviewObject,
	BackParticle,
	Vehicle,
	Effect,
	Projectile,
	Monster,
	Npc,
	Player,
	ItemDrop,
	Liquid,
	MiddleParticle,
	ForegroundTile,
	ForegroundTileMod,
	ForegroundEntity,
	ForegroundOverlay,
	FrontParticle,
	Overlay;

	val base = Point(this)

	fun point(offset: Long = 0L): Point {
		return if (offset == 0L)
			base
		else
			Point(this, offset)
	}

	data class Point(val base: RenderLayer, val offset: Long = 0L) : Comparable<Point> {
		override fun compareTo(other: Point): Int {
			var cmp = base.compareTo(other.base)
			if (cmp == 0) cmp = offset.compareTo(other.offset)
			return cmp
		}
	}

	companion object {
		fun tileLayer(isBackground: Boolean, isModifier: Boolean, offset: Long = 0L): Point {
			if (isBackground && isModifier) {
				return BackgroundTileMod.point(offset)
			} else if (isBackground) {
				return BackgroundTile.point(offset)
			} else if (isModifier) {
				return ForegroundTileMod.point(offset)
			} else {
				return ForegroundTile.point(offset)
			}
		}

		private val logger = LogManager.getLogger()
		private val lowercase: ImmutableMap<String, RenderLayer>

		init {
			val builder = ImmutableMap.Builder<String, RenderLayer>()

			for (value in entries) {
				builder.put(value.name.lowercase(), value)
			}

			lowercase = builder.build()
		}

		private fun perform(value: String, symbol: Char): Point {
			val before = value.substringBefore(symbol).lowercase()
			val after = value.substring(before.length)

			val enum = lowercase[before]

			if (enum == null) {
				logger.error("Unknown render layer: $before in $value; assuming BackgroundOverlay")
				return BackgroundOverlay.base
			}

			val num = after.toLongOrNull()

			if (num == null) {
				logger.error("Invalid render layer string: $value; assuming BackgroundOverlay")
				return BackgroundOverlay.base
			}

			return enum.point(num)
		}

		fun parse(value: String): Point {
			if ('+' in value) {
				return perform(value, '+')
			} else if ('-' in value) {
				return perform(value, '-')
			} else {
				val enum = lowercase[value.lowercase()]

				if (enum == null) {
					logger.error("Unknown render layer: $value; assuming BackgroundOverlay")
					return BackgroundOverlay.base
				}

				return enum.base
			}
		}
	}
}

